@@ -2,9 +2,10 @@
 
 // Load modules
 
+const Assert = require('assert');
 const Crypto = require('crypto');
 const Path = require('path');
-const Util = require('util');
+
 const Escape = require('./escape');
 
 
@@ -35,7 +36,7 @@ exports.clone = function (obj, seen) {
 
     if (!Array.isArray(obj)) {
         if (Buffer.isBuffer(obj)) {
-            newObj = new Buffer(obj);
+            newObj = Buffer.from(obj);
         }
         else if (obj instanceof Date) {
             newObj = new Date(obj.getTime());
@@ -70,7 +71,7 @@ exports.clone = function (obj, seen) {
             const descriptor = Object.getOwnPropertyDescriptor(obj, key);
             if (descriptor &&
                 (descriptor.get ||
-                 descriptor.set)) {
+                    descriptor.set)) {
 
                 Object.defineProperty(newObj, key, descriptor);
             }
@@ -88,7 +89,7 @@ exports.clone = function (obj, seen) {
 
 /*eslint-disable */
 exports.merge = function (target, source, isNullOverride /* = true */, isMergeArrays /* = true */) {
-/*eslint-enable */
+    /*eslint-enable */
 
     exports.assert(target && typeof target === 'object', 'Invalid target value: must be an object');
     exports.assert(source === null || source === undefined || typeof source === 'object', 'Invalid source value: must be null, undefined, or an object');
@@ -113,6 +114,10 @@ exports.merge = function (target, source, isNullOverride /* = true */, isMergeAr
     const keys = Object.keys(source);
     for (let i = 0; i < keys.length; ++i) {
         const key = keys[i];
+        if (key === '__proto__') {
+            continue;
+        }
+
         const value = source[key];
         if (value &&
             typeof value === 'object') {
@@ -253,187 +258,32 @@ exports.applyToDefaultsWithShallow = function (defaults, options, keys) {
 
 // Deep object or array comparison
 
-exports.deepEqual = function (obj, ref, options, seen) {
-
-    options = options || { prototype: true };
-
-    const type = typeof obj;
-
-    if (type !== typeof ref) {
-        return false;
-    }
-
-    if (type !== 'object' ||
-        obj === null ||
-        ref === null) {
-
-        if (obj === ref) {                                                      // Copied from Deep-eql, copyright(c) 2013 Jake Luer, jake@alogicalparadox.com, MIT Licensed, https://github.com/chaijs/deep-eql
-            return obj !== 0 || 1 / obj === 1 / ref;        // -0 / +0
-        }
-
-        return obj !== obj && ref !== ref;                  // NaN
-    }
-
-    seen = seen || [];
-    if (seen.indexOf(obj) !== -1) {
-        return true;                            // If previous comparison failed, it would have stopped execution
-    }
-
-    seen.push(obj);
-
-    if (Array.isArray(obj)) {
-        if (!Array.isArray(ref)) {
-            return false;
-        }
-
-        if (!options.part && obj.length !== ref.length) {
-            return false;
-        }
-
-        for (let i = 0; i < obj.length; ++i) {
-            if (options.part) {
-                let found = false;
-                for (let j = 0; j < ref.length; ++j) {
-                    if (exports.deepEqual(obj[i], ref[j], options)) {
-                        found = true;
-                        break;
-                    }
-                }
-
-                return found;
-            }
-
-            if (!exports.deepEqual(obj[i], ref[i], options)) {
-                return false;
-            }
-        }
-
-        return true;
-    }
-
-    if (Buffer.isBuffer(obj)) {
-        if (!Buffer.isBuffer(ref)) {
-            return false;
-        }
-
-        if (obj.length !== ref.length) {
-            return false;
-        }
-
-        for (let i = 0; i < obj.length; ++i) {
-            if (obj[i] !== ref[i]) {
-                return false;
-            }
-        }
-
-        return true;
-    }
-
-    if (obj instanceof Date) {
-        return (ref instanceof Date && obj.getTime() === ref.getTime());
-    }
-
-    if (obj instanceof RegExp) {
-        return (ref instanceof RegExp && obj.toString() === ref.toString());
-    }
-
-    if (options.prototype) {
-        if (Object.getPrototypeOf(obj) !== Object.getPrototypeOf(ref)) {
-            return false;
-        }
-    }
-
-    const keys = Object.getOwnPropertyNames(obj);
-
-    if (!options.part && keys.length !== Object.getOwnPropertyNames(ref).length) {
-        return false;
-    }
-
-    for (let i = 0; i < keys.length; ++i) {
-        const key = keys[i];
-        const descriptor = Object.getOwnPropertyDescriptor(obj, key);
-        if (descriptor.get) {
-            if (!exports.deepEqual(descriptor, Object.getOwnPropertyDescriptor(ref, key), options, seen)) {
-                return false;
-            }
-        }
-        else if (!exports.deepEqual(obj[key], ref[key], options, seen)) {
-            return false;
-        }
-    }
-
-    return true;
-};
-
-
-// Remove duplicate items from array
-
-exports.unique = (array, key) => {
-
-    let result;
-    if (key) {
-        result = [];
-        const index = new Set();
-        array.forEach((item) => {
-
-            const identifier = item[key];
-            if (!index.has(identifier)) {
-                index.add(identifier);
-                result.push(item);
-            }
-        });
-    }
-    else {
-        result = Array.from(new Set(array));
-    }
-
-    return result;
-};
-
-
-// Convert array into object
-
-exports.mapToObject = function (array, key) {
-
-    if (!array) {
-        return null;
-    }
-
-    const obj = {};
-    for (let i = 0; i < array.length; ++i) {
-        if (key) {
-            if (array[i][key]) {
-                obj[array[i][key]] = true;
-            }
-        }
-        else {
-            obj[array[i]] = true;
-        }
-    }
-
-    return obj;
-};
+exports.deepEqual = require('./deep-equal');
 
 
 // Find the common unique items in two arrays
 
 exports.intersect = function (array1, array2, justFirst) {
 
-    if (!array1 || !array2) {
-        return [];
+    if (!array1 ||
+        !array2) {
+
+        return (justFirst ? null : []);
     }
 
     const common = [];
-    const hash = (Array.isArray(array1) ? exports.mapToObject(array1) : array1);
-    const found = {};
-    for (let i = 0; i < array2.length; ++i) {
-        if (hash[array2[i]] && !found[array2[i]]) {
+    const hash = (Array.isArray(array1) ? new Set(array1) : array1);
+    const found = new Set();
+    for (const value of array2) {
+        if (internals.has(hash, value) &&
+            !found.has(value)) {
+
             if (justFirst) {
-                return array2[i];
+                return value;
             }
 
-            common.push(array2[i]);
-            found[array2[i]] = true;
+            common.push(value);
+            found.add(value);
         }
     }
 
@@ -441,6 +291,16 @@ exports.intersect = function (array1, array2, justFirst) {
 };
 
 
+internals.has = function (ref, key) {
+
+    if (typeof ref.has === 'function') {
+        return ref.has(key);
+    }
+
+    return ref[key] !== undefined;
+};
+
+
 // Test if the reference contains the values
 
 exports.contain = function (ref, values, options) {
@@ -467,7 +327,6 @@ exports.contain = function (ref, values, options) {
 
     options = options || {};            // deep, once, only, part
 
-    exports.assert(arguments.length >= 2, 'Insufficient arguments');
     exports.assert(typeof ref === 'string' || typeof ref === 'object', 'Reference must be string or an object');
     exports.assert(values.length, 'Values array cannot be empty');
 
@@ -638,121 +497,30 @@ exports.reachTemplate = function (obj, template, options) {
 };
 
 
-exports.formatStack = function (stack) {
-
-    const trace = [];
-    for (let i = 0; i < stack.length; ++i) {
-        const item = stack[i];
-        trace.push([item.getFileName(), item.getLineNumber(), item.getColumnNumber(), item.getFunctionName(), item.isConstructor()]);
-    }
-
-    return trace;
-};
-
-
-exports.formatTrace = function (trace) {
-
-    const display = [];
-
-    for (let i = 0; i < trace.length; ++i) {
-        const row = trace[i];
-        display.push((row[4] ? 'new ' : '') + row[3] + ' (' + row[0] + ':' + row[1] + ':' + row[2] + ')');
-    }
-
-    return display;
-};
-
-
-exports.callStack = function (slice) {
-
-    // http://code.google.com/p/v8/wiki/JavaScriptStackTraceApi
-
-    const v8 = Error.prepareStackTrace;
-    Error.prepareStackTrace = function (_, stack) {
-
-        return stack;
-    };
-
-    const capture = {};
-    Error.captureStackTrace(capture, this);     // arguments.callee is not supported in strict mode so we use this and slice the trace of this off the result
-    const stack = capture.stack;
-
-    Error.prepareStackTrace = v8;
-
-    const trace = exports.formatStack(stack);
-
-    return trace.slice(1 + slice);
-};
-
-
-exports.displayStack = function (slice) {
-
-    const trace = exports.callStack(slice === undefined ? 1 : slice + 1);
-
-    return exports.formatTrace(trace);
-};
-
-
-exports.abortThrow = false;
-
-
-exports.abort = function (message, hideStack) {
-
-    if (process.env.NODE_ENV === 'test' || exports.abortThrow === true) {
-        throw new Error(message || 'Unknown error');
-    }
-
-    let stack = '';
-    if (!hideStack) {
-        stack = exports.displayStack(1).join('\n\t');
-    }
-    console.log('ABORT: ' + message + '\n\t' + stack);
-    process.exit(1);
-};
-
-
-exports.assert = function (condition /*, msg1, msg2, msg3 */) {
+exports.assert = function (condition, ...args) {
 
     if (condition) {
         return;
     }
 
-    if (arguments.length === 2 && arguments[1] instanceof Error) {
-        throw arguments[1];
+    if (args.length === 1 && args[0] instanceof Error) {
+        throw args[0];
     }
 
-    let msgs = [];
-    for (let i = 1; i < arguments.length; ++i) {
-        if (arguments[i] !== '') {
-            msgs.push(arguments[i]);            // Avoids Array.slice arguments leak, allowing for V8 optimizations
-        }
-    }
+    const msgs = args
+        .filter((arg) => arg !== '')
+        .map((arg) => {
 
-    msgs = msgs.map((msg) => {
+            return typeof arg === 'string' ? arg : arg instanceof Error ? arg.message : exports.stringify(arg);
+        });
 
-        return typeof msg === 'string' ? msg : msg instanceof Error ? msg.message : exports.stringify(msg);
+    throw new Assert.AssertionError({
+        message: msgs.join(' ') || 'Unknown error',
+        actual: false,
+        expected: true,
+        operator: '==',
+        stackStartFunction: exports.assert
     });
-
-    throw new Error(msgs.join(' ') || 'Unknown error');
-};
-
-
-exports.Timer = function () {
-
-    this.ts = 0;
-    this.reset();
-};
-
-
-exports.Timer.prototype.reset = function () {
-
-    this.ts = Date.now();
-};
-
-
-exports.Timer.prototype.elapsed = function () {
-
-    return Date.now() - this.ts;
 };
 
 
@@ -791,35 +559,6 @@ exports.escapeRegex = function (string) {
 };
 
 
-// Base64url (RFC 4648) encode
-
-exports.base64urlEncode = function (value, encoding) {
-
-    exports.assert(typeof value === 'string' || Buffer.isBuffer(value), 'value must be string or buffer');
-    const buf = (Buffer.isBuffer(value) ? value : new Buffer(value, encoding || 'binary'));
-    return buf.toString('base64').replace(/\+/g, '-').replace(/\//g, '_').replace(/\=/g, '');
-};
-
-
-// Base64url (RFC 4648) decode
-
-exports.base64urlDecode = function (value, encoding) {
-
-    if (typeof value !== 'string') {
-
-        return new Error('Value not a string');
-    }
-
-    if (!/^[\w\-]*$/.test(value)) {
-
-        return new Error('Invalid character');
-    }
-
-    const buf = new Buffer(value, 'base64');
-    return (encoding === 'buffer' ? buf : buf.toString(encoding || 'binary'));
-};
-
-
 // Escape attribute value for use in HTTP header
 
 exports.escapeHeaderAttribute = function (attribute) {
@@ -838,22 +577,9 @@ exports.escapeHtml = function (string) {
 };
 
 
-exports.escapeJavaScript = function (string) {
+exports.escapeJson = function (string) {
 
-    return Escape.escapeJavaScript(string);
-};
-
-
-exports.nextTick = function (callback) {
-
-    return function () {
-
-        const args = arguments;
-        process.nextTick(() => {
-
-            callback.apply(null, args);
-        });
-    };
+    return Escape.escapeJson(string);
 };
 
 
@@ -864,78 +590,22 @@ exports.once = function (method) {
     }
 
     let once = false;
-    const wrapped = function () {
+    const wrapped = function (...args) {
 
         if (!once) {
             once = true;
-            method.apply(null, arguments);
+            method(...args);
         }
     };
 
     wrapped._hoekOnce = true;
-
     return wrapped;
 };
 
 
-exports.isInteger = function (value) {
-
-    return (typeof value === 'number' &&
-            parseFloat(value) === parseInt(value, 10) &&
-            !isNaN(value));
-};
-
-
 exports.ignore = function () { };
 
 
-exports.inherits = Util.inherits;
-
-
-exports.format = Util.format;
-
-
-exports.transform = function (source, transform, options) {
-
-    exports.assert(source === null || source === undefined || typeof source === 'object' || Array.isArray(source), 'Invalid source object: must be null, undefined, an object, or an array');
-    const separator = (typeof options === 'object' && options !== null) ? (options.separator || '.') : '.';
-
-    if (Array.isArray(source)) {
-        const results = [];
-        for (let i = 0; i < source.length; ++i) {
-            results.push(exports.transform(source[i], transform, options));
-        }
-        return results;
-    }
-
-    const result = {};
-    const keys = Object.keys(transform);
-
-    for (let i = 0; i < keys.length; ++i) {
-        const key = keys[i];
-        const path = key.split(separator);
-        const sourcePath = transform[key];
-
-        exports.assert(typeof sourcePath === 'string', 'All mappings must be "." delineated strings');
-
-        let segment;
-        let res = result;
-
-        while (path.length > 1) {
-            segment = path.shift();
-            if (!res[segment]) {
-                res[segment] = {};
-            }
-            res = res[segment];
-        }
-        segment = path.shift();
-        res[segment] = exports.reach(source, sourcePath, options);
-    }
-
-    return result;
-};
-
-
 exports.uniqueFilename = function (path, extension) {
 
     if (extension) {
@@ -951,10 +621,10 @@ exports.uniqueFilename = function (path, extension) {
 };
 
 
-exports.stringify = function () {
+exports.stringify = function (...args) {
 
     try {
-        return JSON.stringify.apply(null, arguments);
+        return JSON.stringify.apply(null, args);
     }
     catch (err) {
         return '[Cannot display object: ' + err.message + ']';
@@ -962,14 +632,13 @@ exports.stringify = function () {
 };
 
 
-exports.shallow = function (source) {
+exports.wait = function (timeout) {
 
-    const target = {};
-    const keys = Object.keys(source);
-    for (let i = 0; i < keys.length; ++i) {
-        const key = keys[i];
-        target[key] = source[key];
-    }
+    return new Promise((resolve) => setTimeout(resolve, timeout));
+};
 
-    return target;
+
+exports.block = function () {
+
+    return new Promise(exports.ignore);
 };
